/*
Article "Digital VU meter with OLED display. Arduino project for beginners":
http://www.techn0man1ac.info/arduino/diy-digital-vumeter-oled-and-arduino
Video "Arduino project for beginners. VU meter with OLED display. Techn0man1ac
https://youtu.be/qoLsynwssLI
This project is taken as a basis:
https://forum.arduino.cc/index.php?topic=403234.0

By Techn0man1ac
*/

#include "display.c"
#include <OLED_I2C.h> // Library: OLED_I2C http://www.rinkydinkelectronics.com/library.php?id=79

 /*
Board I2C / TWI pins
Uno, Ethernet A4 (SDA), A5 (SCL)
Mega2560  20 (SDA), 21 (SCL)
Leonardo  2 (SDA), 3 (SCL)
Due 20 (SDA), 21 (SCL), SDA1, SCL1
*/
OLED  myOLED(SDA, SCL, 8);

#define analogInput 0 // This line sets the analogue pin Arduino;
#define HighSens true // high sensitivity mode, achieved by changing the reference voltage in 1.1 V, 
//can be either enabled true or disabled - false, I recommend to enable;

#define FASTADC true // acceleration of the ADC, for Arduino Nano / Uno / Pro Mini it is recommended to enable(true);
#define Sensitivity 1024 // sensitivity, this value cannot be more than 1024 if your audio source 
//gives a small output voltage, then you can try to reduce this value by 2, 4 times;

#define SampleWindow 15  // number of samples, number in milliseconds, the smaller it is, the faster the arrow twitches, 
//the default is 50, but I picked up 15-20 for myself, it’s no longer pulling the arduin.

// defines for setting and clearing register bits
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

//extern uint8_t VUMeter[];

int hMeter = 65; // horizontal center for needle animation
int vMeter = 85; // vertical center for needle animation (outside of dislay limits)
int rMeter = 80; // length of needle animation or arch of needle travel

int sample;

void setup() {

  if (HighSens == true) // Enable (true) / disable (false) high sensitivity mode
{  
    analogReference(INTERNAL);  //an built-in reference, equal to 1.1 volts on the ATmega168 or ATmega328 
    // and 2.56 volts on the ATmega8 (not available on the Arduino Mega)
    
    //analogReference(INTERNAL1V1); //INTERNAL1V1: a built-in 1.1V reference (Arduino Mega only)
  } else {
    analogReference(DEFAULT);
  }

#if FASTADC
  // set prescale to 16
  sbi(ADCSRA, ADPS2) ;
  cbi(ADCSRA, ADPS1) ;
  cbi(ADCSRA, ADPS0) ;
#endif

  myOLED.begin(); // needed for SH1106 display
  myOLED.clrScr(); // clears display from any library info displayed
}

void loop() {

  unsigned long startMillis = millis();  // start of sample window
  unsigned int PeaktoPeak = 0;        // peak-to-peak level
  unsigned int SignalMax = 0;
  unsigned int SignalMin = Sensitivity;

  while ( millis() - startMillis < SampleWindow ) 
  {

    sample = analogRead(analogInput);
    if (sample < Sensitivity) 
    {

      if (sample > SignalMax) 
      {

        SignalMax = sample;  // saves just the max levels
      }

      else if (sample < SignalMin) 
      {

        SignalMin = sample;  // saves just the min levels
      }
    }
  }

  PeaktoPeak = SignalMax - SignalMin; // max - min = peak-peak amplitude
  float MeterValue = PeaktoPeak * 330 / Sensitivity;     // convert volts to arrow information

  MeterValue = MeterValue - 34;       // shifts needle to zero position
  //myOLED.clrScr();  // refresh display for next step
  myOLED.drawBitmap(0, 0, VUMeter, 128, 64);       // draws background
  int a1 = (hMeter + (sin(MeterValue / 57.296) * rMeter)); // meter needle horizontal coordinate
  int a2 = (vMeter - (cos(MeterValue / 57.296) * rMeter)); // meter needle vertical coordinate
  myOLED.drawLine(a1, a2, hMeter, vMeter);         // draws needle
  myOLED.update();
}
